from utils import StringUtils
from java.io import InputStreamReader,OutputStreamWriter,IOException
from java.util import InputMismatchException
from java.net import Socket
from MyStrategy import MyStrategy
import sys
import traceback

class Runner:
    def __init__(self, socket, strategy, token):
        self.socket = socket
        self.input = InputStreamReader(socket.getInputStream())
        self.output = OutputStreamWriter(socket.getOutputStream())
        self.strategy = strategy
        self.token = token

    def disconnect(self):
        try:
            self.socket.close()
        except IOException:
            traceback.print_exc()

    def run(self):
        try:
            self.output.write(self.token)
            self.output.flush()
            
            while True:
                cr = self.input.read()
                if cr == -1:
                    break
                else:
                    c = chr(cr)
                    if c == 'D':
                        self.disconnect()
                    elif c == 'A':
                        player = StringUtils.readPlayer(self.input)
                        world = StringUtils.readWorld(self.input)
                        a = self.strategy.getAction(player, world)
                        StringUtils.writeToken(self.output, a)
                        self.output.flush()
                    elif c == 'I':
                        rules = StringUtils.readRules(self.input)
                        self.strategy.init(rules)
                    else:
                        raise InputMismatchException, "Unknown action " + c
        except Exception:
            traceback.print_exc()
            self.disconnect()
        

def main(argv):
    host = "127.0.0.1" if len(argv)<2 else argv[1]
    port = 7828 if len(argv)<3 else int(argv[2])
    token = "0000000000000000000000000000000000000000000000000000000000000000" if len(argv)<4 else argv[3]
   
    socket = Socket(host, port)
    runner = Runner(socket, MyStrategy(), token)
    runner.run()

if __name__ == "__main__":
    main(sys.argv)
